#!/usr/bin/env python
# -*- coding:utf8 -*-

import re
import os
import markdown
from pygments import highlight
from pygments.lexers import get_lexer_by_name, TextLexer
from pygments.formatters import HtmlFormatter


def render(ns, NameSpace):
    raw_posts = os.listdir(ns.site.content)
    os.chdir(ns.site.content)
    for post in raw_posts:
        print 'reading %s' % post
        container = NameSpace()
        container.meta = NameSpace()

        raw_post = open(post).read().decode('utf-8')
        meta_pattern = re.compile(r'======.*?\======', re.S)
        metas = meta_pattern.findall(raw_post)[0].split('\n')
        #get meta
        for meta in metas:
            try:
                key, value = meta.split(':')
                container.meta[key.strip()] = value.strip()
            except:
                pass
        #preview meta
        if container.meta.get('preview', 'False').upper() == 'FALSE':
            container.meta.preview = False
        else:
            container.meta.preview = True
        #public meta
        if container.meta.get('public', 'True').upper() == 'TRUE':
            container.meta.public = True
        else:
            container.meta.public = False
        #folder meta
        if container.meta.get('folder', None):
            ns.context.folder.append(container.meta.folder)
        container.meta.link = os.path.splitext(post)[0] + '.html'
        #tag meta
        try:
            container.meta.tags = container.meta.tags.split(',')
        except AttributeError:
            container.meta.tags = None
        finally:
            if container.meta.tags:
                container.meta.tags = list(set(container.meta.tags))
        #date meta
        year, month, day = container.meta.date.split('-')
        month = month if len(month) == 2 else '0' + month
        day = day if len(day) == 2 else '0' + day
        container.meta.date = '%s-%s-%s' % (year, month, day)

        #mkd render
        md = markdown.Markdown(
            safe_mode=False,
            output_format='xhtml',
        )
        raw_content = raw_post.split('======')[2]
        container.content = md.convert(hilite(raw_content))
        ns.context.posts.append(container)
    ns.context.folder = list(set(ns.context.folder))

    #sort posts
    ns.context.posts.sort(lambda x, y: cmp(y.meta.date, x.meta.date))

    public_posts = [p for p in calc_public_posts(ns.context.posts)]
    ns.context.public_posts = public_posts
    preview_posts = [p for p in calc_preview_posts(public_posts)]
    ns.context.preview_posts = preview_posts


def hilite(raw_post):
    #pygments render
    formatter = HtmlFormatter(noclasses=False)
    pyg_pattern = re.compile(r'~~~(\w+)(.*?)~~~', re.S)

    def repl(m):
        try:
            lexer = get_lexer_by_name(m.group(1))
        except:
            lexer = TextLexer()
        code = highlight(m.group(2), lexer, formatter)
        return code

    return pyg_pattern.sub(repl, raw_post)


def calc_public_posts(posts):
    for p in posts:
        if p.meta.public:
            yield p


def calc_preview_posts(posts):
    for p in posts:
        if p.meta.preview:
            yield p



#!/usr/bin/env python
# -*- coding:utf8 -*-

import os
import datetime
from jinja2 import Environment, FileSystemLoader


def write(ns):
    #load templates
    print 'loading templates...'
    tpl_dir = os.path.join(ns.root.path, '_templates/%s') % ns.context.theme
    jinja = Environment(
        loader=FileSystemLoader(tpl_dir, encoding='utf-8'),
        autoescape=False,
    )
    jinja.filters['atomdatetime'] = atom_date_time
    index_tpl = jinja.get_template('index.html')
    post_tpl = jinja.get_template('post.html')
    feed_tpl = jinja.get_template('feed.xml')
    #write index.html
    print 'writing index.html'
    index = index_tpl.render(
        site=ns.site,
        context=ns.context,
        public_posts=ns.context.public_posts,
        preview_posts=ns.context.preview_posts,
        hilite=u'home',
    )
    desti = os.path.join(ns.site.deploy, 'index.html')
    f = open(desti, 'w')
    f.write(index.encode('utf-8'))
    f.close()
    #writer feed.xml
    feed = feed_tpl.render(
        site=ns.site,
        public_posts=ns.context.public_posts,
        preview_posts=ns.context.preview_posts,
        now=datetime.datetime.now()
    )
    desti = os.path.join(ns.site.deploy, 'feed.xml')
    f = open(desti, 'w')
    f.write(feed.encode('utf-8'))
    f.close()

    #write all posts
    for post in ns.context.public_posts:
        print 'writing %s' % post.meta.link
        result = post_tpl.render(
            site=ns.site,
            context=ns.context,
            post=post,
            hilite='home')
        desti = os.path.join(ns.site.deploy, post.meta.link)
        f = open(desti, 'w')
        f.write(result.encode('utf-8'))
        f.close()

        #make folder
        if post.meta.folder:
            folder = os.path.join(ns.site.deploy, post.meta.folder)
            if not os.path.exists(folder):
                print 'making folder %s...' % folder
                os.mkdir(folder)
        else:
            print '%s "folder" key missing...' % post.meta.link
    #deploy folder
    folder_list = list(set(ns.context.folder))
    for f in folder_list:
        folded_posts = []
        print 'deploying folder %s...' % f
        for post in ns.context.public_posts:
            if post.meta.folder == f:
                folded_posts.append(post)
        result = index_tpl.render(
            site=ns.site,
            context=ns.context,
            public_posts=folded_posts,
            hilite=f
        )
        desti_folder = os.path.join(ns.site.deploy, f)
        index = os.path.join(desti_folder, 'index.html')
        f = open(index, 'w')
        f.write(result.encode('utf-8'))
        f.close()


def atom_date_time(value, format='%Y-%m-%dT%H:%M:%SZ'):
    if isinstance(value, datetime.datetime):
        return value.strftime(format)
    year, month, day = value.split('-')
    value = datetime.datetime.now().replace(
        year=int(year),
        month=int(month),
        day=int(day)
    ).strftime(format)
    return value



#!/usr/bin/env python
# -*- coding:utf8 -*-

import os
import shutil

from sblog import __version__ as version
from namespace import ns, NameSpace
from utils import timer
import readers
import writers
import ConfigParser



def init():
    print 'Sblog version %s' % version
    ns.root.path = os.path.dirname(__file__)
    ns.root.version = version
    ns.root.template_dir = os.path.join(os.path.dirname(__file__),
                                        '_templates')
    ns.root.static_dir = os.path.join(os.path.dirname(__file__), '_static')
    ns.site.proj_dir = os.getcwd()


def pre_read():
    cf = ConfigParser.ConfigParser()
    config = os.path.join(ns.site.proj_dir, 'config.ini')
    cf.read(config)

    ns.site.site_name = cf.get('basic', 'site_name').decode('utf-8')
    ns.site.author = cf.get('basic', 'author').decode('utf-8')
    ns.site.site_url = cf.get('basic', 'site_url').decode('utf-8')
    ns.site.content = os.path.join(os.getcwd(), 'content')
    ns.site.deploy = os.path.join(os.getcwd(), 'deploy')
    try:
        ns.context.ga = cf.get('extra', 'ga')
    except:
        ns.context.ga = None
    try:
        ns.context.disqus = cf.get('extra', 'disqus')
    except:
        ns.context.disqus = None
    try:
        ns.context.theme = cf.get('extra','theme')
    except:
        ns.context.theme = 'default'


def build():
    if 'config.ini' in os.listdir('.'):
        pre_read()
        read()
        write()
        gene_static()
    else:
        s = raw_input("build an environment?(y/n):")
        if s != 'n':
            shutil.copyfile(os.path.join(ns.root.path, 'config.ini'),
                            'config.ini')
            os.mkdir('content')
            os.mkdir('deploy')
            print 'build finished'
            return


def read():
    print 'reading...'
    readers.render(ns, NameSpace)


def write():
    print 'writing...'
    writers.write(ns)


def gene_static():
    desti_static = os.path.join(ns.site.deploy, 'static')
    if os.path.exists(desti_static):
        print 'removing old static files...'
        shutil.rmtree(desti_static)
    print 'generating new static files...'
    shutil.copytree(ns.root.static_dir, desti_static)

@timer()
def main():
    init()
    build()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
install_requires = ['markdown', 'Jinja2', 'pygments']


from sblog import __version__ as version

setup(
    name='sblog',
    version=version,
    author='xiaoba',
    author_email='xiaobayuwei@gmail.com',
    url='http://blog.xiaoba.me',
    packages=['sblog', 'sblog.writers', 'sblog.readers'],
    description='A lightweight static weblog generator',
    long_description=open('README.mkd').read(),
    license='BSD License',
    entry_points={
        'console_scripts': ['sblog= sblog.main:main'],
    },
    install_requires=install_requires,
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Programming Language :: Python',
    ]
)



#!/usr/bin/env python
# -*- coding:utf8 -*-

import functools
import datetime


class timer(object):
    def __call__(self, method):
        @functools.wraps(method)
        def wrapper(*args, **kwargs):
            start = datetime.datetime.now()
            print 'start at %s...' % start
            method(*args, **kwargs)
            end = datetime.datetime.now()
            print 'finish at %s...' % end
            #print 'using %s...' % (end - start).seconds
            print 'using %s...' % (end - start)
        return wrapper



#!/usr/bin/env python
# -*- coding:utf8 -*-


class NameSpace(dict):
    def __getattr__(self, key):
        if key in self:
            return self[key]
        else:
            raise AttributeError
    def __setattr__(self, key, value):
        self[key] = value
    def __delattr__(self, key):
        try:
            del self[key]
        except:
            raise AttributeError


ns = NameSpace()
ns.root = NameSpace()
ns.site = NameSpace()
ns.context = NameSpace()
ns.context.folder = []
ns.context.posts = []
ns.context.pages = []
